@@ -25,7 +25,7 @@
     :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
     :copyright: Copyright 2011-2014 Marc Rautenhaus (mr), Omar Qunsul (oq)
     :copyright: Copyright 2016-2017 Reimar Bauer
-    :copyright: Copyright 2016-2021 by the mss team, see AUTHORS.
+    :copyright: Copyright 2016-2022 by the mss team, see AUTHORS.
     :license: APACHE-2.0, see LICENSE for details.
 
     Licensed under the Apache License, Version 2.0 (the "License");
@@ -45,25 +45,30 @@ from future import standard_library
 
 standard_library.install_aliases()
 
+import glob
 import os
 import io
+import inspect
 import logging
+import shutil
+import tempfile
 import traceback
 import urllib.parse
-import inspect
+
 from xml.etree import ElementTree
 from chameleon import PageTemplateLoader
 from owslib.crs import axisorder_yx
 from PIL import Image
-import shutil
-
+import numpy as np
 from flask import request, make_response, render_template
 from flask_httpauth import HTTPBasicAuth
+
 from multidict import CIMultiDict
 from mslib.utils import conditional_decorator
-from mslib.utils import parse_iso_datetime
+from mslib.utils.time import parse_iso_datetime
 from mslib.index import app_loader
-from mslib.mswms.gallery_builder import add_image, write_html, write_doc_index, STATIC_LOCATION, DOCS_LOCATION
+from mslib.mswms.gallery_builder import add_image, write_html, add_levels, add_times, \
+    write_doc_index, write_code_pages, STATIC_LOCATION, DOCS_LOCATION
 
 # Flask basic auth's documentation
 # https://flask-basicauth.readthedocs.io/en/latest/#flask.ext.basicauth.BasicAuth.check_credentials
@@ -132,7 +137,7 @@ if mss_wms_settings.__dict__.get('enable_basic_http_authentication', False):
         return authfunc(username, password)
 
 from mslib.mswms import mss_plot_driver
-from mslib.utils import get_projection_params
+from mslib.utils.coordinate import get_projection_params
 
 # Logging the Standard Output, which will be added to the Apache Log Files
 logging.basicConfig(level=logging.DEBUG,
@@ -215,7 +220,7 @@ class WMSServer(object):
                 self.register_lsec_layer(layer[1], layer_class=layer[0])
 
     def generate_gallery(self, create=False, clear=False, generate_code=False, sphinx=False, plot_list=None,
-                         all_plots=False, url_prefix=""):
+                         all_plots=False, url_prefix="", levels="", itimes="", vtimes="", simple_naming=False):
         """
         Iterates through all registered layers, draws their plots and puts them in the gallery
         """
@@ -227,33 +232,31 @@ class WMSServer(object):
                 dataset = [next(iter(mss_wms_settings.data))]
                 mss_wms_settings.register_horizontal_layers = [
                     (plot[1], dataset) for plot in inspect.getmembers(mpl_hsec_styles, inspect.isclass)
-                    if not any(x in plot[0] or x in str(plot[1]) for x in ["Abstract", "Target", "fnord"])
+                    if plot[0] != "HS_GenericStyle" and
+                    not any(x in plot[0] or x in str(plot[1]) for x in ["Abstract", "Target", "fnord"])
                 ]
                 mss_wms_settings.register_vertical_layers = [
                     (plot[1], dataset) for plot in inspect.getmembers(mpl_vsec_styles, inspect.isclass)
-                    if not any(x in plot[0] or x in str(plot[1]) for x in ["Abstract", "Target", "fnord"])
+                    if plot[0] != "VS_GenericStyle" and
+                    not any(x in plot[0] or x in str(plot[1]) for x in ["Abstract", "Target", "fnord"])
                 ]
                 mss_wms_settings.register_linear_layers = [
                     (plot[1], dataset) for plot in inspect.getmembers(mpl_lsec_styles, inspect.isclass)
                 ]
                 self.__init__()
 
-            location = DOCS_LOCATION if sphinx else STATIC_LOCATION
-            if clear and os.path.exists(os.path.join(location, "plots")):
-                shutil.rmtree(os.path.join(location, "plots"))
-            if os.path.exists(os.path.join(location, "code")):
-                shutil.rmtree(os.path.join(location, "code"))
-            if os.path.exists(os.path.join(location, "plots.html")):
-                os.remove(os.path.join(location, "plots.html"))
-
             if not (create or generate_code or all_plots or plot_list):
                 return
 
-            if not plot_list:
+            tmp_path = tempfile.mkdtemp()
+            path = DOCS_LOCATION if sphinx else STATIC_LOCATION
+
+            if plot_list is None:
                 plot_list = [[self.lsec_drivers, self.lsec_layer_registry],
                              [self.vsec_drivers, self.vsec_layer_registry],
                              [self.hsec_drivers, self.hsec_layer_registry]]
 
+            # Iterate through all plots of all datasets, create the plot and build the gallery with it
             for driver, registry in plot_list:
                 multiple_datasets = len(driver) > 1
                 for dataset in driver:
@@ -266,62 +269,171 @@ class WMSServer(object):
                             "Side" if driver == self.vsec_drivers else "Top"
 
                         try:
-                            if not os.path.exists(os.path.join(location, "plots",
-                                                               f"{l_type}_{dataset if multiple_datasets else ''}"
-                                                               f"{plot_object.name}.png")):
-                                # Plot doesn't already exist, generate it
-                                file_types = [field[0] for field in plot_object.required_datafields
-                                              if field[0] != "sfc"]
-                                file_type = file_types[0] if file_types else "sfc"
-                                init_time = plot_driver.get_init_times()[-1]
-                                valid_time = plot_driver.get_valid_times(plot_object.required_datafields[0][1],
-                                                                         file_type, init_time)[-1]
-                                style = plot_object.styles[0][0] if plot_object.styles else None
-                                kwargs = {"plot_object": plot_object,
-                                          "init_time": init_time,
-                                          "valid_time": valid_time}
-                                if driver == self.lsec_drivers:
-                                    plot_driver.set_plot_parameters(**kwargs, lsec_path=[[0, 0, 20000], [1, 1, 20000]],
-                                                                    lsec_numpoints=201, lsec_path_connection="linear")
-                                    path = [[min(plot_driver.lat_data), min(plot_driver.lon_data), 20000],
-                                            [max(plot_driver.lat_data), max(plot_driver.lon_data), 20000]]
-                                    plot_driver.update_plot_parameters(lsec_path=path)
-                                elif driver == self.vsec_drivers:
-                                    plot_driver.set_plot_parameters(**kwargs, vsec_path=[[0, 0], [1, 1]],
-                                                                    vsec_numpoints=201, figsize=[800, 600],
-                                                                    vsec_path_connection="linear", style=style,
-                                                                    noframe=False, bbox=[101, 1050, 10, 180])
-                                    path = [[min(plot_driver.lat_data), min(plot_driver.lon_data)],
-                                            [max(plot_driver.lat_data), max(plot_driver.lon_data)]]
-                                    plot_driver.update_plot_parameters(vsec_path=path)
-                                elif driver == self.hsec_drivers:
-                                    elevations = plot_object.get_elevations()
-                                    elevation = float(elevations[len(elevations) // 2]) if len(elevations) > 0 else None
-                                    plot_driver.set_plot_parameters(**kwargs, noframe=False, figsize=[800, 600],
-                                                                    crs="EPSG:4326", style=style,
-                                                                    bbox=[-15, 35, 30, 65],
-                                                                    level=elevation)
-                                    bbox = [min(plot_driver.lon_data), min(plot_driver.lat_data),
-                                            max(plot_driver.lon_data), max(plot_driver.lat_data)]
-                                    # Create square bbox for better images
-                                    # if abs(bbox[0] - bbox[2]) > abs(bbox[1] - bbox[3]):
-                                    #     bbox[2] = bbox[0] + abs(bbox[1] - bbox[3])
-                                    # else:
-                                    #     bbox[3] = bbox[1] + abs(bbox[0] - bbox[2])
-                                    plot_driver.update_plot_parameters(bbox=bbox)
-                                add_image(plot_driver.plot(), plot_object, generate_code, sphinx, url_prefix=url_prefix,
-                                          dataset=dataset if multiple_datasets else "")
-                            else:
-                                # Plot already exists, skip generation
-                                add_image(None, plot_object, generate_code, sphinx, url_prefix=url_prefix,
-                                          dataset=dataset if multiple_datasets else "")
+                            file_types = [field[0] for field in plot_object.required_datafields
+                                          if field[0] != "sfc"]
+                            file_type = file_types[0] if file_types else "sfc"
+
+                            # All specified init times, or the latest if empty, or all if "all",
+                            # or None if there are no init times
+                            init_times = [parse_iso_datetime(itime) if isinstance(itime, str) else itime
+                                          for itime in (itimes.split(",") if itimes != "all" and itimes != "" else
+                                                        plot_driver.get_init_times() if itimes == "all" else
+                                                        [plot_driver.get_init_times()[-1]])] or [None]
+
+                            for itime in sorted(init_times):
+                                if itime and plot_driver.get_init_times() and itime not in plot_driver.get_init_times():
+                                    logging.warning(f"Requested itime {itime} not present for "
+                                                    f"{dataset} {plot_object.name}! itimes present: "
+                                                    f"{plot_driver.get_init_times()}")
+                                    continue
+                                elif not plot_driver.get_init_times():
+                                    itime = None
+
+                                try:
+                                    # All valid times for the specific init time
+                                    i_vtimes = plot_driver.get_valid_times(plot_object.required_datafields[0][1],
+                                                                           file_type, itime)
+                                except IndexError:
+                                    # ToDo fix demodata for sfc
+                                    logging.debug("plot_object.required_datafields incomplete"
+                                                  " for filetype: %s in dataset: %s for l_type: %s",
+                                                  file_type, dataset, l_type)
+                                    continue
+
+                                # All specified valid times, or the latest if empty, or all if "all",
+                                # or None if there are no valid times for the init time
+                                valid_times = [parse_iso_datetime(vtime) if isinstance(vtime, str) else vtime
+                                               for vtime in (vtimes.split(",") if vtimes != "all" and vtimes != "" else
+                                                             i_vtimes if vtimes == "all" else [i_vtimes[-1]])] or [None]
+
+                                for vtime in sorted(valid_times):
+                                    if vtime and i_vtimes and vtime not in i_vtimes:
+                                        logging.warning(f"Requested vtime {vtime} at {itime} not present for "
+                                                        f"{dataset} {plot_object.name}! vtimes present: {i_vtimes}")
+                                        continue
+                                    elif not i_vtimes:
+                                        vtime = None
+
+                                    style = plot_object.styles[0][0] if plot_object.styles else None
+                                    kwargs = {"plot_object": plot_object,
+                                              "init_time": itime,
+                                              "valid_time": vtime}
+                                    filename = f"{l_type}_{dataset if multiple_datasets else ''}{plot_object.name}-" \
+                                               + f"Noneit{itime}vt{vtime}".replace(" ", "_").replace(":", "_")\
+                                                                          .replace("-", "_")
+
+                                    exists = (not clear) and os.path.exists(
+                                        os.path.join(path, "plots", filename + ".png"))
+
+                                    if driver == self.lsec_drivers and not exists:
+                                        plot_driver.set_plot_parameters(**kwargs,
+                                                                        lsec_path=[[0, 0, 20000], [1, 1, 20000]],
+                                                                        lsec_numpoints=201,
+                                                                        lsec_path_connection="linear")
+                                        lon_data = np.rad2deg(np.unwrap(np.deg2rad(plot_driver.lon_data)))
+                                        lpath = [[min(plot_driver.lat_data), min(lon_data), 20000],
+                                                 [max(plot_driver.lat_data), max(lon_data), 20000]]
+                                        plot_driver.update_plot_parameters(lsec_path=lpath)
+
+                                    elif driver == self.vsec_drivers and not exists:
+                                        plot_driver.set_plot_parameters(**kwargs, vsec_path=[[0, 0], [1, 1]],
+                                                                        vsec_numpoints=201, figsize=[800, 600],
+                                                                        vsec_path_connection="linear", style=style,
+                                                                        noframe=False, bbox=[101, 1050, 10, 180])
+                                        lon_data = np.rad2deg(np.unwrap(np.deg2rad(plot_driver.lon_data)))
+                                        lpath = [[min(plot_driver.lat_data), min(lon_data)],
+                                                 [max(plot_driver.lat_data), max(lon_data)]]
+                                        plot_driver.update_plot_parameters(vsec_path=lpath)
+
+                                    elif driver == self.hsec_drivers:
+                                        elevations = plot_object.get_elevations()
+                                        elevation = float(elevations[len(elevations) // 2]) if len(elevations) > 0 \
+                                            else None
+                                        plot_driver.set_plot_parameters(**kwargs, noframe=False, figsize=[800, 600],
+                                                                        crs="EPSG:4326", style=style,
+                                                                        bbox=[-15, 35, 30, 65],
+                                                                        level=elevation)
+                                        lon_data = np.rad2deg(np.unwrap(np.deg2rad(plot_driver.lon_data)))
+                                        bbox = [min(lon_data), min(plot_driver.lat_data),
+                                                max(lon_data), max(plot_driver.lat_data)]
+                                        vert_units = plot_driver.vert_units
+                                        plot_driver.update_plot_parameters(bbox=bbox)
+
+                                        # All specified levels, or the mid if empty, or all if "all",
+                                        # or None if there are no levels
+                                        rendered_levels = [float(elev) if elev else elev for elev in
+                                                           (levels.split(",") if (levels != "all" and levels != "") else
+                                                            elevations if levels == "all" else [elevation])] or [None]
+
+                                        for level in sorted(rendered_levels):
+                                            if level and elevations and level \
+                                                    not in [float(elev) for elev in elevations]:
+                                                logging.warning(f"Requested level {level} not present for "
+                                                                f"{dataset} {plot_object.name}! Levels present: "
+                                                                f"{elevations}")
+                                                continue
+                                            elif not elevations:
+                                                level = None
+
+                                            plot_driver.update_plot_parameters(level=level)
+
+                                            # filename is different for top layers, so needs a redefine
+                                            filename = f"{l_type}_{dataset if multiple_datasets else ''}" \
+                                                       f"{plot_object.name}-" + \
+                                                       f"{level}{vert_units}it{itime}vt{vtime}".replace(" ", "_")\
+                                                                                               .replace(":", "_")\
+                                                                                               .replace("-", "_")
+                                            exists = (not clear) and os.path.exists(
+                                                os.path.join(path, "plots", filename + ".png"))
+
+                                            add_image(tmp_path, None if exists else plot_driver.plot(), plot_object,
+                                                      generate_code, sphinx, url_prefix=url_prefix,
+                                                      dataset=dataset if multiple_datasets else "", level=f"{level}" +
+                                                                                                f"{vert_units}",
+                                                      itime=str(itime), vtime=str(vtime), simple_naming=simple_naming)
+                                            if level:
+                                                add_levels([f"{level} {vert_units}"], vert_units)
+                                            else:
+                                                add_levels(["None None"], "None")
+
+                                            add_times(itime, [vtime])
+                                        continue
+
+                                    add_image(
+                                        tmp_path,
+                                        None if exists else plot_driver.plot(), plot_object, generate_code,
+                                        sphinx, url_prefix=url_prefix,
+                                        dataset=dataset if multiple_datasets else "", itime=str(itime),
+                                        vtime=str(vtime), simple_naming=simple_naming)
+                                    add_times(itime, [vtime])
 
                         except Exception as e:
                             traceback.print_exc()
                             logging.error("%s %s %s", plot_object.name, type(e), e)
-            write_html(sphinx)
-            if sphinx and generate_code:
-                write_doc_index()
+
+            write_html(tmp_path, sphinx)
+            if generate_code:
+                write_code_pages(tmp_path, sphinx, url_prefix)
+                if sphinx:
+                    write_doc_index(tmp_path)
+
+            if clear and os.path.exists(os.path.join(path, "plots")):
+                shutil.rmtree(os.path.join(path, "plots"))
+            if os.path.exists(os.path.join(path, "plots")):
+                for fn in glob.glob(os.path.join(tmp_path, "plots", "*")):
+                    if not os.path.exists(os.path.join(path, "plots", os.path.basename(fn))):
+                        shutil.move(fn, os.path.join(path, "plots"))
+            else:
+                shutil.move(os.path.join(tmp_path, "plots"), path)
+            if os.path.exists(os.path.join(path, "code")):
+                shutil.rmtree(os.path.join(path, "code"))
+            if os.path.exists(os.path.join(tmp_path, "code")):
+                shutil.move(os.path.join(tmp_path, "code"), path)
+            if os.path.exists(os.path.join(path, "plots.html")):
+                os.remove(os.path.join(path, "plots.html"))
+            if os.path.exists(os.path.join(tmp_path, "plots.html")):
+                shutil.move(os.path.join(tmp_path, "plots.html"), path)
+            shutil.rmtree(tmp_path)
 
     def register_hsec_layer(self, datasets, layer_class):
         """
@@ -349,6 +461,12 @@ class WMSServer(object):
                     self.hsec_layer_registry[dataset] = {}
                 else:
                     raise ValueError("dataset '%s' not available", dataset)
+            if layer.name in self.hsec_layer_registry[dataset]:
+                raise ValueError(f"new layer is already registered? dataset={dataset} layer.name={layer.name} "
+                                 f"new={layer} old={self.hsec_layer_registry[dataset][layer.name]}")
+            if layer.name == "HS_GenericStyle":
+                raise ValueError(f"problem in configuration for dataset={dataset}. We found layer.name={layer.name}"
+                                 f"  The class HS_GenericStyle should never be used.")
             self.hsec_layer_registry[dataset][layer.name] = layer
 
     def register_vsec_layer(self, datasets, layer_class):
@@ -374,6 +492,12 @@ class WMSServer(object):
                     self.vsec_layer_registry[dataset] = {}
                 else:
                     raise ValueError("dataset '%s' not available", dataset)
+            if layer.name in self.vsec_layer_registry[dataset]:
+                raise ValueError(f"new layer is already registered? dataset={dataset} layer.name={layer.name} "
+                                 f"new={layer} old={self.vsec_layer_registry[dataset][layer.name]}")
+            if layer.name == "VS_GenericStyle":
+                raise ValueError(f"problem in configuration for dataset={dataset}. We found layer.name={layer.name}"
+                                 f" The class VS_GenericStyle should never be used.")
             self.vsec_layer_registry[dataset][layer.name] = layer
 
     def register_lsec_layer(self, datasets, variable=None, filetype="ml", layer_class=None):
@@ -402,6 +526,9 @@ class WMSServer(object):
                     self.lsec_layer_registry[dataset] = {}
                 else:
                     raise ValueError("dataset '%s' not available", dataset)
+            if layer.name in self.lsec_layer_registry[dataset]:
+                raise ValueError(f"new layer is already registered? dataset={dataset} layer.name={layer.name} "
+                                 f"new={layer} old={self.lsec_layer_registry[dataset][layer.name]}")
             self.lsec_layer_registry[dataset][layer.name] = layer
 
     def create_service_exception(self, code=None, text="", version="1.3.0"):
@@ -646,11 +773,12 @@ class WMSServer(object):
                 level = query.get('ELEVATION')
                 level = float(level) if level is not None else None
                 layer_datatypes = self.hsec_layer_registry[dataset][layer].required_datatypes()
-                if level is None and any(_x in layer_datatypes for _x in ["pl", "al", "ml", "tl", "pv"]):
+                from mslib.utils.netCDF4tools import VERTICAL_AXIS
+                if level is None and any(_x in layer_datatypes for _x in VERTICAL_AXIS):
                     # Use the default value.
                     level = -1
                 elif ("sfc" in layer_datatypes) and \
-                        all(_x not in layer_datatypes for _x in ["pl", "al", "ml", "tl", "pv"]) and \
+                        all(_x not in layer_datatypes for _x in VERTICAL_AXIS) and \
                         level is not None:
                     return self.create_service_exception(
                         text=f"ELEVATION argument not applicable for layer '{layer}'. Please omit this argument.",
